﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }
    }

    public class SimpleDataCollection : ObservableCollection<SimpleDataPoint>
    {
        public SimpleDataCollection()
        {
            this.Add(new SimpleDataPoint() { X = 1.0, Y = 3.0 });
            this.Add(new SimpleDataPoint() { X = 2.0, Y = 2.0 });
            this.Add(new SimpleDataPoint() { X = 3.0, Y = 3.0 });
            this.Add(new SimpleDataPoint() { X = 4.0, Y = 4.0 });
            this.Add(new SimpleDataPoint() { X = 5.0, Y = 5.0 });
            this.Add(new SimpleDataPoint() { X = 6.0, Y = 6.0 });
            this.Add(new SimpleDataPoint() { X = 7.0, Y = 5.0 });
        }
    }

    /// <summary>
    /// Simple storage class for pair of double values
    /// </summary>
    public class SimpleDataPoint
    {
        public double X { get; set; }
        public double Y { get; set; }
    }

    public class LineSeries : Series
    {
        public LineSeries()
        {
            this.DefaultStyleKey = typeof(LineSeries);
        }

        public const string XAxisPropertyName = "XAxis";
        public const string YAxisPropertyName = "YAxis";
        public const string ValuePropertyName = "Value";

        /// <summary>
        /// The value of the line to draw
        /// </summary>
        public static readonly DependencyProperty ValueProperty = DependencyProperty.Register(
        ValuePropertyName,
        typeof(double),
        typeof(LineSeries),
        new PropertyMetadata(0D));

        public double Value
        {
            get
            {
                return (double)this.GetValue(ValueProperty);
            }
            set
            {
                this.SetValue(ValueProperty, value);
                RaisePropertyChanged(ValuePropertyName, 0, value);
            }
        }

        public static readonly DependencyProperty XAxisProperty = DependencyProperty.Register(
        XAxisPropertyName,
        typeof(NumericXAxis),
        typeof(LineSeries),
        new PropertyMetadata(null, (sender, e) =>
        {
            LineSeries series = (LineSeries)sender;
            series.RaisePropertyChanged(XAxisPropertyName, e.OldValue, e.NewValue);
        }
        )
        );

        public NumericXAxis XAxis
        {
            get
            {
                return this.GetValue(XAxisProperty) as NumericXAxis;
            }
            set
            {
                this.SetValue(XAxisProperty, value);
            }
        }

        public static readonly DependencyProperty YAxisProperty = DependencyProperty.Register(
        YAxisPropertyName,
        typeof(NumericYAxis),
        typeof(LineSeries),
        new PropertyMetadata(null, (sender, e) =>
        {
            LineSeries series = (LineSeries)sender;
            series.RaisePropertyChanged(YAxisPropertyName, e.OldValue, e.NewValue);
        }
        )
        );

        public NumericYAxis YAxis
        {
            get
            {
                return this.GetValue(YAxisProperty) as NumericYAxis;
            }
            set
            {
                this.SetValue(YAxisProperty, value);
            }
        }

        /// <summary>
        /// Calls rendering of this series any time the Viewport rect has changed.
        /// </summary>
        /// <param name="oldViewportRect"></param>
        /// <param name="newViewportRect"></param>
        protected override void ViewportRectChangedOverride(Rect oldViewportRect, Rect newViewportRect)
        {
            base.ViewportRectChangedOverride(oldViewportRect, newViewportRect);
            this.RenderSeries(false);
        }

        /// <summary>
        /// Calls rendering of this series any time the Window rect has changed.
        /// </summary>
        /// <param name="oldWindowRect"></param>
        /// <param name="newWindowRect"></param>
        protected override void WindowRectChangedOverride(Rect oldWindowRect, Rect newWindowRect)
        {
            base.WindowRectChangedOverride(oldWindowRect, newWindowRect);
            this.RenderSeries(false);
        }

        /// <summary>
        /// Checks if series should be redrawn any time a property of the series has changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="propertyName"></param>
        /// <param name="oldValue"></param>
        /// <param name="newValue"></param>
        protected override void PropertyUpdatedOverride(object sender, string propertyName, object oldValue, object newValue)
        {
            base.PropertyUpdatedOverride(sender, propertyName, oldValue, newValue);
            Console.WriteLine(propertyName);
            switch (propertyName)
            {
                //renders series on changes made to the ranges
                case ValuePropertyName:
                    this.RenderSeries(false);
                    if (this.XAxis != null)
                    {
                        this.XAxis.UpdateRange();
                    }
                    if (this.YAxis != null)
                    {
                        this.YAxis.UpdateRange();
                    }
                    break;

                //renders seris if a new X-axis is assigned
                case XAxisPropertyName:
                    if (oldValue != null)
                    {
                        ((Axis)oldValue).DeregisterSeries(this);
                    }
                    if (newValue != null)
                    {
                        ((Axis)newValue).RegisterSeries(this);
                    }
                    if ((XAxis != null && !XAxis.UpdateRange()) || (newValue == null && oldValue != null))
                    {
                        RenderSeries(false);
                    }
                    break;

                //renders series if a new Y-axis is assigned
                case YAxisPropertyName:
                    if (oldValue != null)
                    {
                        ((Axis)oldValue).DeregisterSeries(this);
                    }
                    if (newValue != null)
                    {
                        ((Axis)newValue).RegisterSeries(this);
                    }
                    if ((YAxis != null && !YAxis.UpdateRange()) || (newValue == null && oldValue != null))
                    {
                        RenderSeries(false);
                    }
                    break;
            }
        }

        /// <summary>
        /// Calculates range of a given axis based on the X/Y values of the data items
        /// </summary>
        /// <param name="axis"></param>
        /// <returns></returns>
        protected override AxisRange GetRange(Axis axis)
        {

            //Set the minimum range for the Axis to include the point where the line is drawn
            if (axis == this.YAxis)
            {
                return new AxisRange(this.Value, this.Value);
            }
            else
            {
                //Since there is no axis range needed to display this series, return null.
                return null;
            }
        }

        protected override void RenderSeriesOverride(bool animate)
        {
            //disable series rendering with transitions (Motion Framework)
            base.RenderSeriesOverride(animate);

            //check to see if the series can be rendered
            if (this.Viewport.IsEmpty || this.RootCanvas == null || this.XAxis == null || this.YAxis == null)
            {
                return;
            }

            //clear the RootCanvas on every render of the series
            this.RootCanvas.Children.Clear();

            Grid grid = new Grid();

            double x1 = this.Viewport.Left;
            double x2 = this.Viewport.Right;
            double y1 = this.YAxis.GetScaledValue(this.Value, this.SeriesViewer.WindowRect, this.Viewport);
            double y2 = y1;
            Line line = new Line();
            line.StrokeThickness = 4;
            line.Stroke = Brushes.Red;
            line.X1 = x1;
            line.Y1 = y1;
            line.X2 = x2;
            line.Y2 = y2;
            grid.Children.Add(line);

            Canvas.SetZIndex(grid, 0);
            this.RootCanvas.Children.Add(grid);
        }
    }

}
